%{
	#include "scanner.h"
	#include "Driver.h"
	#define yyterminate() return token::END
%}

%option noyywrap nounput batch c++ yyclass="FlexScanner"


ID		[a-z][A-Za-z0-9_]*
VARIABLE	[A-Z][A-Za-z0-9_]*
STRING		\"([^\"]|\\\")*\"
NUMBER		"0"|[1-9][0-9]*

HIDE	"#hide"[^\.]*\.
SHOW	"#show"[^\.]*\.
METASTATEMENT	{SHOW}|{HIDE}

CHAR_LEXEM	[\_\.\,\?\:\;\|\+\-\*\/\@\(\)\[\]\{\}\=\<\>\\\&\?\^\~]


COMMENT		"%"([^*\n][^\n]*)?\n
MULTI_LINE_COMMENT	"%*"([^*]|\*[^%])*"*%"
BLANK		[ \t]+

%{
	#define YY_USER_ACTION yylloc->columns(yyleng);
%}

%%

%{
	yylloc->step();
%}

([\r\n]|[\n])+		{ yylloc->lines(yyleng); yylloc->step(); }

{COMMENT}		{ yylloc->step(); }
{MULTI_LINE_COMMENT}	{ std::string s = std::string(yytext, yyleng); yylloc->lines(std::count(s.begin(), s.end(), '\n') /*number of newlines*/); yylloc->step(); }
{BLANK}			{ yylloc->step(); }

%{
	typedef parser::Parser::token token;
	typedef parser::Parser::token_type token_type;
%}

"not"		return token::NAF;
":-"		return token::CONS;
":~"		return token::WCONS;
"<>"|"!="	return token::UNEQUAL;
"<="		return token::LESS_OR_EQ;
">="		return token::GREATER_OR_EQ;
"**"		return token::EXP;

"#minimi"[zs]"e"	return token::MINIMIZE;
"#maximi"[zs]"e"	return token::MAXIMIZE;

"#count"	return token::AGGREGATE_COUNT;
"#max"		return token::AGGREGATE_MAX;
"#min"		return token::AGGREGATE_MIN;
"#sum"		return token::AGGREGATE_SUM;

".."		return token::INTERVAL;

{CHAR_LEXEM}	return token_type(yytext[0]);

{ID}		{ yylval->string = new std::string(yytext, yyleng); return token::ID; }
{VARIABLE}	{ yylval->string = new std::string(yytext, yyleng); return token::VARIABLE; }
{STRING}	{ yylval->string = new std::string(yytext, yyleng); return token::STRING; }
{NUMBER}	{ yylval->string = new std::string(yytext, yyleng); return token::NUMBER; }

{METASTATEMENT}	{ yylval->string = new std::string(yytext, yyleng); return token::METASTATEMENT; }

.		{ return static_cast<token_type>(*yytext); }

%%

