#ifndef LOGICPROGRAMOPTIMIZATION_H_
#define LOGICPROGRAMOPTIMIZATION_H_

#include <sharp/main>
using namespace sharp;

#include "../LogicProgramProblem.hpp"
using namespace lpopt;

#include "../asp-domain/asp-domain.hpp"

namespace lpopt
{
	class LogicProgramOptimizationTuple : public Tuple
	{
	public:
		LogicProgramOptimizationTuple();
		virtual ~LogicProgramOptimizationTuple();

	public:
		ClassicalLiteral *temporary;
		set<string> ignoreHeadTags; // if head vairalbes were ignored during decomposition, this set contains all those variables that had and have to be tagged along to the root node as temporary predicates' arguments. otherwise NULL/irrelevant.

	public:
		virtual bool operator<(const Tuple &other) const;
		virtual bool operator==(const Tuple &other) const;
		virtual int hash() const;
		
	};

	class LogicProgramOptimizationAlgorithm : public AbstractHTDAlgorithm
	{
	public:
		LogicProgramOptimizationAlgorithm(LogicProgramProblem *problem);
		virtual ~LogicProgramOptimizationAlgorithm();
		void printBenchmarks() const; // prints my captured benchmarks.

	protected:
		virtual Solution *selectSolution(TupleSet *tuples, const ExtendedHypertree *node); // ANALYSIS: prints output
		virtual TupleSet *evaluateNode(const ExtendedHypertree *node);

	protected:
		LogicProgramProblem *problem;
		clock_t bodyPart_time, extractBody_time, bodyCreation_time, tempPredCreation_time, unsafe_time, rest_time;
		domain::Statement* workingStatement;
		domain::varset* originalStatementsVars;
	};
} // lpopt
#endif
